<?php
/**
 * The file that defines the METABOX class
 *
 * @link       http://brewpixel.com
 * @since      1.0.1
 *
 * @package    BrewPress
 */

namespace BrewPixel\BrewPress\MetaBox;

use BrewPixel\BrewPress\Session\includes\WPSession;
use BrewPixel\BrewPress\Session\Session;

/**
 *
 * ** Field Types **
 *
 * @TODO Complete metaboxes for all these types
 * [x] Text (type text, api returns text)
 * [x] Text Area (type text, api returns text)
 * [x] File (upload a file, api returns the url)
 * [x] Select (drop down list of choices, api returns chosen item)
 * [x] Checkbox (tickbox list of choices, api returns array of choices)
 * [x] Radio Buttons ( radio button list of choices, api returns chosen item)
 * [x] Date Picker (jquery date picker, options for format, api returns string)
 * [x] Color Picker (WP color swatch picker)
 * [x] WYSIWYG Editor (a wordpress wysiwyg editor, api returns html)
 * [x] Number (type number, api returns integer)
 * [x] Currency (@TODO: Multi-currency)
 * [ ] True / False (tick box with message, api returns true or false)
 * [ ] Email (type email, api returns text)
 * [ ] Password (type password, api returns text)
 * [x] Image (upload an image, api returns the image attachment id)
 * [ ] Page Link (select 1 or more page, post or custom post types, api returns the selected url)
 * [ ] Post Object (select 1 or more page, post or custom post types, api returns the selected post objects)
 * [ ] Relationship (search, select and order post objects with a tidy interface, api returns the selected post objects)
 * [ ] Taxonomy (select taxonomy terms with options to load, display and save, api returns the selected term objects)
 * [ ] User (select 1 or more WP users, api returns the selected user objects)
 * [ ] Google Maps (interactive map, api returns lat,lng,address data)
 * [ ] Tab (Group fields into tabs)
 * [ ] Message (Render custom messages into the fields)
 * [ ] Repeater (ability to create repeatable blocks of fields!)
 * [ ] Flexible Content (ability to create flexible blocks of fields!)
 * [ ] Gallery (Add, edit and order multiple images in 1 simple field)
 * [ ] oEmbed (The oEmbed field allows an easy way to embed videos, images, tweets, audio, and other content. This field makes use of the native WP oEmbed functionality.
 * [ ] Custom (Create your own field type!)
 *
 * Class MetaBox
 * @package BrewPixel\BrewPress\MetaBox\Metabox
 */
class Metabox {

	/**
	 * The name of the post type.
	 * @var string
	 */
	public $post_type_name;

	/**
	 * The title of the metabox.
	 *
	 * @var string
	 */
	public $title;

	/**
	 * Where is the metabox located? Side or.
	 *
	 * @var string
	 */
	public $location_context;

	/**
	 * What is the priority of the position? High, ?
	 *
	 * @var string
	 */
	public $priority;

	/**
	 * Additional form fields for the metabox.
	 *
	 * @var array
	 */
	public $form_fields = array();

	/**
	 * The tanonomy array.
	 *
	 * @var array
	 */
	public $tax = array();

	/**
	 * The id names.
	 *
	 * @var array
	 */
	public $id_names = array();

	/**
	 * Allow js?
	 *
	 * @var bool
	 */
	private $js;

	/**
	 * Allow html?
	 *
	 * @var bool
	 */
	private $allowhtml;

	public $divid;

	/**
	 * Constructor for the Metabox class.
	 *
	 * @param string $name Name of metabox, this is also the ID of it.
	 * @param string $title Title of metabox. This is displayed.
	 * @param string $location_context The location of the metabox (side, core).
	 * @param string $priority The priority or how high to the top it should be
	 *                         in the location it is.
	 * @param array $form_fields Various additional arguments.
	 */
	public function __construct(
		$name,
		$title,
		$location_context,
		$priority,
		$form_fields = array()
	) {
		if ( ! defined( 'WP_SESSION_COOKIE' ) ) {
			define( 'WP_SESSION_COOKIE', 'bp_wp_session' );
		}

		$this->post_type_name   = strtolower( $name );
		$this->title            = $title;
		$this->location_context = $location_context;
		$this->priority         = $priority;
		$this->form_fields      = $form_fields;

		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ) );
		add_action( 'save_post', array( $this, 'save_post' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'brewpixel_enqueue_admin_javascript' ) );
	}

	/**
	 * @return array
	 */
	public function getTax() {
		return $this->tax;
	}

	/**
	 * @param array $tax
	 */
	public function setTax( $tax ) {
		if ( is_array( $tax ) ) {
			$this->tax = $tax;
		} else {
			$this->tax = array( $tax );
		}
	}

	/**
	 * Helper method, that attaches a passed function to the 'admin_init' WP
	 * action
	 *
	 * @param callable $cb Passed callback function.
	 */
	public function admin_init( $cb ) {
		add_action( 'admin_init', $cb );
	}

	/**
	 * Creates a new custom meta box in the New 'post_type' page.
	 *
	 * Adds the meta box container.
	 *
	 * @param string $post_type The post type name (post, page, bpdirector, etc.).
	 */
	public function add_meta_box( $post_type ) {
		$post_types = array( $this->post_type_name );     // Limit meta box to certain post types.
		if ( in_array( $post_type, $post_types ) ) {
			add_meta_box(
				strtolower( str_replace( ' ', '_', $this->title ) ), // ID.
				$this->title,
				array( $this, 'render_meta_box_content' ),
				$this->post_type_name, // Associated post type.
				$this->location_context, // Location/context. normal, side, etc.
				$this->priority, // Priority level default, 10, 20 etc.
				array( $this->form_fields ) // Optional passed arguments.
			);
		}
	}

	/**
	 * Output the HTML of the metabox area. Display the form fields.
	 *
	 * @param object $post The post.
	 * @param object $data The data.
	 */
	public function render_meta_box_content( $post, $data ) {
		global $post;
		wp_nonce_field( plugin_basename( __FILE__ ), 'BP_nonce' );
		add_action( 'post_edit_form_tag',
			function() {
				echo ' enctype="multipart/form-data"';
			} );

		// List of all the specified form fields.
		$inputs = $data['args'][0];
		// Get the saved field values.
		$meta = get_post_custom( $post->ID );
		// For each form field specified, we need to create the necessary markup.
		// $name = Label, $type = the type of input to create.
		$divid        = $data['id'];
		foreach ( $inputs as $name => $type ) {

			if ( isset( $inputs[ $name ]['field_type'] ) && $inputs[ $name ]['field_type'] === 'grouprepeater' ) {
				foreach ( $inputs[ $name ]['sub_fields'] as $subname => $subtype ) {
					/*
				 * Example: 'Happiness Info' in 'Snippet Info' box becomes:
				 * snippet_info_happiness_info.
				 */
					$id_name    = $data['id'] . '_' .
					              strtolower(
						              str_replace(
							              ' ',
							              '_',
							              $subname
						              )
					              ) . '['.$name.']';
					$id_name_meta    = $data['id'] . '_' .
					                   strtolower(
						                   str_replace(
							                   ' ',
							                   '_',
							                   $subname
						                   )
					                   );
					$taxarray   = $this->getTax();
					$taxarray[] = $id_name;
					$this->setTax( $taxarray );

					$link_id_name = $data['id'] . '_' .
					                strtolower( str_replace( ' ', '_', $subname ) );
					// If this is an array then it is either a file upload, a select drop-down or a link.
					if ( is_array( $inputs[ $name ]['sub_fields'][ $subname ] ) ) {
						if ( 'link' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) ) {
							$link = $this->render_link_form_field( $inputs[ $name ]['sub_fields'], $subname, $meta, $id_name );
						} elseif ( 'select' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) ) {
							$select = $this->render_select_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta );
						} elseif ( 'radio' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) ) {
							$radio = $this->render_radio_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta );
						} elseif ( 'textwithdescription' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) || 'text' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) ) {
							$textWithDescription = $this->render_text_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype );
						} elseif ( 'textarea' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) ) {
							$textarea = $this->render_textarea_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype );
						} elseif ( 'date' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) ) {
							$date = $this->render_date_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype );
						} elseif ( 'image' === strtolower( $inputs[ $name ]['sub_fields'][ $subname ]['field_type'] ) ) {
							$date = $this->render_image_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype );
						}
					}

					// Attempt to set the value of the input, based on what's saved in the db.
					$value         = isset( $meta[ $id_name_meta ][0] ) ? $meta[ $id_name_meta ][0] : '';
					$esc_valuimage  = esc_html( $value );
					$now           = strtotime( 'now' );
					$esc_date      = ( '' != $value ? $esc_value : '' );
					$timestamp     = strtotime( $esc_date );
					$esc_date_nice = ( '' != $value ? date( 'l, d M, Y', $timestamp ) : '' );
					// TODO - Add the other input types.
					$lookup = array(
						'hiddentext'          => "<input type='text' name='".$id_name."[]' value='$esc_value' />",
						'timestamp'           => "<input id='save-date' type='hidden' name='".$id_name."[]' value='$now' class='widefat' />",
						'text'                => $this->render_text_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype, true ),
						'text_name'           => $this->render_text_name_form_field( $id_name, $esc_value, $post->ID ),
						'date'                => $this->render_date_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype ),
						'url'                 => $this->render_url_form_field( $id_name, $esc_value ),
						'textarea'            => $this->render_textarea_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype ),
						'number'              => $this->render_number_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype ),
						'currency'            => $this->render_currency_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype ),
						'image'               => $this->render_image_form_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype, $divid ),
						'link'                => isset( $link ) ? $link : '',
						'person_name'         => isset( $person_name ) ? $person_name : '',
						'select'              => isset( $select ) ? $select : '',
						'radio'               => isset( $radio ) ? $radio : '',
						'textWithDescription' => isset( $textWithDescription ) ? $textWithDescription : '',
						'file'                => $this->render_file_form_field( $post->ID, $id_name, $subtype ),
						'checkbox'            => $this->render_checkbox_form_field( $id_name, $subname, $esc_value, $subtype ),
						'wpeditor'            => $this->render_wpeditor_wysiwyg_field( $id_name, $inputs[ $name ]['sub_fields'], $subname, $meta, $subtype ),
					);
					$this->display_html( $subtype, $subname, $lookup );
					var_dump('$name');
					var_dump($name);
					var_dump('$subname');
					var_dump($subname);
					var_dump('$type');
					var_dump($type);
					var_dump('$subtype');
					var_dump($subtype);
					var_dump($lookup);
					var_dump($id_name);
					var_dump($inputs[ $name ]['sub_fields']);
					var_dump($meta);
					var_dump($divid);
					var_dump($esc_value);
					//die();
				}
			} else {

				/*
				 * Example: 'Happiness Info' in 'Snippet Info' box becomes:
				 * snippet_info_happiness_info.
				 */
				$id_name    = $data['id'] . '_' .
				              strtolower(
					              str_replace(
						              ' ',
						              '_',
						              $name
					              )
				              );
				$taxarray   = $this->getTax();
				$taxarray[] = $id_name;
				$this->setTax( $taxarray );

				$link_id_name = $data['id'] . '_' .
				                strtolower( str_replace( ' ', '_', $name ) );
				// If this is an array then it is either a file upload, a select drop-down or a link.
				if ( is_array( $inputs[ $name ] ) ) {
					if ( 'link' === strtolower( $inputs[ $name ]['field_type'] ) ) {
						$link = $this->render_link_form_field( $inputs, $name, $meta, $id_name );
					} elseif ( 'select' === strtolower( $inputs[ $name ]['field_type'] ) ) {
						$select = $this->render_select_form_field( $id_name, $inputs, $name, $meta );
					} elseif ( 'radio' === strtolower( $inputs[ $name ]['field_type'] ) ) {
						$radio = $this->render_radio_form_field( $id_name, $inputs, $name, $meta );
					} elseif ( 'textwithdescription' === strtolower( $inputs[ $name ]['field_type'] ) || 'text' === strtolower( $inputs[ $name ]['field_type'] ) ) {
						$textWithDescription = $this->render_text_form_field( $id_name, $inputs, $name, $meta, $type );
					} elseif ( 'textarea' === strtolower( $inputs[ $name ]['field_type'] ) ) {
						$textarea = $this->render_textarea_form_field( $id_name, $inputs, $name, $meta, $type );
					} elseif ( 'date' === strtolower( $inputs[ $name ]['field_type'] ) ) {
						$date = $this->render_date_form_field( $id_name, $inputs, $name, $meta, $type );
					} elseif ( 'image' === strtolower( $inputs[ $name ]['field_type'] ) ) {
						$image = $this->render_image_form_field( $id_name, $inputs, $name, $meta, $type );
					}
				}

				// Attempt to set the value of the input, based on what's saved in the db.
				$value         = isset( $meta[ $id_name ][0] ) ? $meta[ $id_name ][0] : '';
				$esc_value     = esc_html( $value );
				$now           = strtotime( 'now' );
				$esc_date      = ( '' !== $value ? $esc_value : '' );
				$timestamp     = strtotime( $esc_date );
				$esc_date_nice = ( '' !== $value ? date( 'l, d M, Y', $timestamp ) : '' );
				// TODO - Add the other input types.
				$lookup = array(
					'hiddentext'          => "<input type='text' name='$id_name' value='$esc_value' />",
					'timestamp'           => "<input id='save-date' type='hidden' name='$id_name' value='$now' class='widefat' />",
					'text'                => $this->render_text_form_field( $id_name, $inputs, $name, $meta, $type ),
					'text_name'           => $this->render_text_name_form_field( $id_name, $esc_value, $post->ID ),
					'date'                => $this->render_date_form_field( $id_name, $inputs, $name, $meta, $type ),
					'url'                 => $this->render_url_form_field( $id_name, $esc_value ),
					'textarea'            => $this->render_textarea_form_field( $id_name, $inputs, $name, $meta, $type ),
					'number'              => $this->render_number_form_field( $id_name, $inputs, $name, $meta, $type ),
					'currency'            => $this->render_currency_form_field( $id_name, $inputs, $name, $meta, $type ),
					'image'               => $this->render_image_form_field( $id_name, $inputs, $name, $meta, $type, $divid ),
					'link'                => isset( $link ) ? $link : '',
					'person_name'         => isset( $person_name ) ? $person_name : '',
					'select'              => isset( $select ) ? $select : '',
					'radio'               => isset( $radio ) ? $radio : '',
					'textWithDescription' => isset( $textWithDescription ) ? $textWithDescription : '',
					'file'                => $this->render_file_form_field( $post->ID, $id_name, $type ),
					'checkbox'            => $this->render_checkbox_form_field( $id_name, $name, $esc_value, $type ),
					'wpeditor'            => $this->render_wpeditor_wysiwyg_field( $id_name, $inputs, $name, $meta, $type ),
				);
				$this->display_html( $type, $name, $lookup );
			}
		}
	}

	private function render_wpeditor_wysiwyg_field( $id_name, $inputs, $name, $meta, $type ) {
		$html = '';
		if ( isset($inputs[ $name ]['field_type']) && 'wpeditor' === $inputs[ $name ]['field_type'] ) {
			$content   = isset( $meta[ $id_name ][0] ) ? $meta[ $id_name ][0] : '';
			$editor_id = $id_name;
			if ( is_array( $type ) ) {
				$settings = $type;
			} else {
				$settings = array();
			}
			ob_start();
			wp_editor( $content, $editor_id, $settings );
			$html            = ob_get_clean();
			$this->allowhtml = ( array_key_exists( 'allowhtml',
				$inputs[ $name ] ) && $inputs[ $name ]['allowhtml'] ? true : false );
		}
		return $html;
	}

	private function render_text_form_field( $id_name, $inputs, $name = null, $meta = null, $type = null, $grouprepeat = false ) {

		$value   = isset( $meta[ $id_name ][0] ) ? esc_attr( $meta[ $id_name ][0] ) : '';
		if ( $grouprepeat ) {
			$id_name = esc_attr( $id_name )."[$name]".'[]';
			//$id_name = esc_attr( $id_name );
		} else {
			$id_name = esc_attr( $id_name );
		}

		if ( is_array( $type ) ) {
			$this->allowhtml = ( array_key_exists( 'allowhtml',
					$inputs[ $name ] ) && $inputs[ $name ]['allowhtml'] );
			$placeholder = ( array_key_exists( 'placeholder', $inputs[ $name ] ) ? $inputs[ $name ]['placeholder'] : '' );
			$helpertext  = ( array_key_exists( 'helpertext', $inputs[ $name ] ) ? $inputs[ $name ]['helpertext'] : '' );
			$html        = "<input type='text' id='$id_name' name='$id_name' value='" . $value . "' placeholder='" . esc_attr( $placeholder ) . "' class='widefat' /><p class='howto'>" . esc_attr( $helpertext ) . "</p>";
		} else {
			$html = "<input type='text' id='$id_name' name='$id_name' value='$value' class='widefat' />";
		}

		return $html;
	}

	private function render_number_form_field( $id_name, $inputs, $name = null, $meta = null, $type = null ) {

		$value   = isset( $meta[ $id_name ][0] ) ? esc_attr( $meta[ $id_name ][0] ) : '';
		$id_name = esc_attr( $id_name );

		if ( is_array( $type ) ) {
			$this->allowhtml = ( array_key_exists( 'allowhtml',
				$inputs[ $name ] ) && $inputs[ $name ]['allowhtml'] ? true : false );
			$placeholder = ( array_key_exists( 'placeholder', $inputs[ $name ] ) ? $inputs[ $name ]['placeholder'] : '' );
			$helpertext  = ( array_key_exists( 'helpertext', $inputs[ $name ] ) ? $inputs[ $name ]['helpertext'] : '' );
			$html        = "<input type='number' id='$id_name' name='$id_name' value='" . $value . "' placeholder='" . esc_attr( $placeholder ) . "' class='widefat' /><p class='howto'>" . esc_attr( $helpertext ) . "</p>";
		} else {
			$html = "<input type='number' id='$id_name' name='$id_name' value='$value' class='widefat' />";
		}

		return $html;
	}

	private function render_currency_form_field( $id_name, $inputs, $name = null, $meta = null, $type = null ) {
		//$price = number_format( $number, 2, '.', ',' );
		//setlocale(LC_MONETARY, 'en_GB');
		//money_format('%.2n', $price;
		$value   = isset( $meta[ $id_name ][0] ) ? esc_attr( $meta[ $id_name ][0] ) : '';
		$id_name = esc_attr( $id_name );

		if ( is_array( $type ) ) {
			$this->allowhtml = ( array_key_exists( 'allowhtml',
				$inputs[ $name ] ) && $inputs[ $name ]['allowhtml'] ? true : false );
			$placeholder = ( array_key_exists( 'placeholder', $inputs[ $name ] ) ? $inputs[ $name ]['placeholder'] : '' );
			$helpertext  = ( array_key_exists( 'helpertext', $inputs[ $name ] ) ? $inputs[ $name ]['helpertext'] : '' );
			$html        = "<input type='number' id='$id_name' name='$id_name' min='0.01' step='0.01' value='" . $value . "' placeholder='" . esc_attr( $placeholder ) . "' class='widefat' /><p class='howto'>" . esc_attr( $helpertext ) . "</p>";
		} else {
			$html = "<input type='number' id='$id_name' name='$id_name' min='0.01' step='0.01' value='$value' class='widefat' />";
		}

		return $html;
	}

	private function render_image_form_field( $id_name, $inputs, $name = null, $meta = null, $type = null, $divid = null ) {
		$value   = isset( $meta[ $id_name ][0] ) ? esc_attr( $meta[ $id_name ][0] ) : '';
		$id_name = esc_attr( $id_name );
		$this->divid = $divid;
		global $content_width, $_wp_additional_image_sizes;
		$image_id = get_post_meta( get_the_ID(), $id_name, true );
		$old_content_width = $content_width;
		$content_width = 254;
		$html = '';
		if ( $image_id && get_post( $image_id ) ) {
			if ( ! isset( $_wp_additional_image_sizes['post-thumbnail'] ) ) {
				$thumbnail_html = wp_get_attachment_image( $image_id, array( $content_width, $content_width ) );
			} else {
				$thumbnail_html = wp_get_attachment_image( $image_id, 'post-thumbnail' );
			}
			if ( ! empty( $thumbnail_html ) ) {
				$html = $thumbnail_html;
				$html .= '<p class="hide-if-no-js"><a href="javascript:;" id="remove_' . esc_attr( $id_name ) . '_button" >' . esc_html__( 'Remove ' . $this->divid . ' image', 'text-domain' ) . '</a></p>';
				$html .= '<input type="hidden" id="' . $id_name . '" name="' . esc_attr( $id_name ) . '" value="' . esc_attr( $image_id ) . '" />';
			}
			$content_width = $old_content_width;
		} else {
			$html = '<img src="" style="width:' . esc_attr( $content_width ) . 'px;height:auto;border:0;display:none;" />';
			$html .= '<p class="hide-if-no-js"><a title="' . esc_attr__( 'Set ' . $this->divid . ' image', 'text-domain' ) . '" href="javascript:;" id="upload_' . esc_attr( $id_name ) . '_button" data-uploader_title="' . esc_attr__( 'Choose an image', 'text-domain' ) . '" data-uploader_button_text="' . esc_attr__( 'Set ' . $this->divid . ' image', 'text-domain' ) . '">' . esc_html__( 'Set ' . $this->divid . ' image', 'text-domain' ) . '</a></p>';
			$html .= '<input type="hidden" id="' . esc_attr( $id_name ) . '" name="' . esc_attr( $id_name ) . '" value="" />';
		}
		$image_array = array( 'imagediv' =>  $this->divid, 'idname' => $id_name,  );
		wp_localize_script( 'brewpress-admin-edit', 'imagemetabox', $image_array );
		return $html;
	}

	public function render_date_form_field( $id_name, $inputs, $name = null, $meta = null, $type = null ) {

		$value   = isset( $meta[ $id_name ][0] ) ? esc_attr( $meta[ $id_name ][0] ) : '';
		$id_name = esc_attr( $id_name );

		$esc_value     = esc_html( $value );
		$esc_date      = ( '' != $value ? $esc_value : '' );
		$timestamp     = strtotime( $esc_date );
		$esc_date_nice = ( '' != $value ? date( 'l, d M, Y', $timestamp ) : '' );

		if ( is_array( $type ) ) {
			$this->allowhtml = ( array_key_exists( 'allowhtml', $inputs[ $name ] ) && $inputs[ $name ]['allowhtml'] ? true : false );
			$placeholder = ( array_key_exists( 'placeholder', $inputs[ $name ] ) ? $inputs[ $name ]['placeholder'] : '' );
			$helpertext  = ( array_key_exists( 'helpertext', $inputs[ $name ] ) ? $inputs[ $name ]['helpertext'] : '' );
			$html = "<input id='datepicker' type='text' name='$id_name' placeholder='Saturday, 16 November, 2013' value='$esc_date_nice' class='widefat' style='position: relative; z-index: 100000;'><input id='actualDate' type='hidden' name='$id_name' value='$esc_date' class='widefat' />";
			//$this->brewpixel_add_actions();
		} else {
			$html = "<input id='datepicker' type='text' name='$id_name' placeholder='Saturday, 16 November, 2013' value='$esc_date_nice' class='widefat' style='position: relative; z-index: 100000;'><input id='actualDate' type='hidden' name='$id_name' value='$esc_date' class='widefat' />";
			//<input type='text' id='$id_name' name='$id_name' value='$value' class='widefat' />
			//$this->admin_init( array( $this, 'brewpixel_add_actions' ) );
		}

		return $html;
	}

	public function brewpixel_add_actions( $cb = null ) {
		echo 'testing2';
		add_action( 'admin_enqueue_scripts', array( $this, 'brewpixel_enqueue_admin_edit_javascript' ) );

	}

	public function brewpixel_enqueue_admin_javascript( $hook ) {
		if ( is_admin() ) {

			if ( $hook !== 'post-new.php' && $hook !== 'post.php' ) {
				return;
			} else {
				wp_enqueue_script( 'jquery-ui-datepicker' );
				wp_enqueue_script( 'jquery-datepicker-brewpixel',
					plugins_url( 'brewpress/public/admin/js/jquery.datepicker.brewpixel.js' ),
					array( 'jquery', 'jquery-ui-core', 'jquery-ui-datepicker' ),
					'2016',
					false
				);
				wp_enqueue_style( 'wp-jquery-ui-datepicker',
					plugins_url( 'brewpress/public/admin/css/jquery-ui-1.12.0.custom/jquery-ui.min.css' ),
					array(),
					'2016'
				);
			}
		}
	}

	public function brewpixel_enqueue_admin_edit_javascript( $hook ) {
		//if ( is_admin() ) {

		//if ( $hook !== 'post-new.php' && $hook !== 'post.php' ) {
		//	return;
		//} else {
		var_dump('late');
		var_dump($this->divid);

		//}
		//}
	}

	private function render_textarea_form_field( $id_name, $inputs, $name = null, $meta = null, $type = null ) {

		$value   = isset( $meta[ $id_name ][0] ) ? esc_attr( $meta[ $id_name ][0] ) : '';
		$id_name = esc_attr( $id_name );
		if ( is_array( $type ) ) {

			$placeholder = ( array_key_exists( 'placeholder', $inputs[ $name ] ) ? $inputs[ $name ]['placeholder'] : '' );
			$helpertext  = ( array_key_exists( 'helpertext', $inputs[ $name ] ) ? $inputs[ $name ]['helpertext'] : '' );
			$this->js    = ( array_key_exists( 'js', $inputs[ $name ] ) && $inputs[ $name ]['js'] ? true : false );
			$html        = "<textarea name='$id_name' class='widefat' rows='10' placeholder='" . esc_attr( $placeholder ) . "'>$value</textarea><p class='howto'>" . esc_attr( $helpertext ) . "</p>";
		} else {
			$html = "<textarea name='$id_name' class='widefat' rows='10'>$value</textarea>";
		}

		return $html;
	}

	private function render_url_form_field( $id_name, $esc_value ) {
		$html = "<div class='meta-link'><input type='text' id='$id_name' name='$id_name' value='" . esc_url( $esc_value ) . "' class='widefat' /></div>";

		return $html;
	}

	private function render_select_form_field( $id_name, $inputs, $name, $meta ) {
		// Filter through them, and create options.
		$select = "<select name='$id_name' class='widefat'>";

		foreach ( $inputs[ $name ][0] as $option ) {
			/*
			 * If what's stored in the db is equal to the
			 * current value in the foreach, that should
			 * be the selected one
			 */
			if ( isset( $meta[ $id_name ] ) && $meta[ $id_name ][0] == $option ) {
				$set_selected = "selected='selected'";
			} else {
				$set_selected = '';
			}

			$select .= "<option value='$option' $set_selected> $option </option>";
		}
		$select .= '</select>';

		return $select;
	}

	private function render_file_form_field( $postID, $id_name, $type ) {
		$file = get_post_meta( $postID, $id_name, true );
		$html = ( isset( $file ) && ! empty( $file ) ) ? "<div class='target-$id_name'><input class='removeImageBtn' type='button' value='Remove Image' /></div><input type='file' name='$id_name' id='$id_name' /><br /><span>&nbsp;Maximum recommended image dimensions 552px width and 506px height.</span>" : '<input type="file" name="$id_name" id="imageurls" /><input type= "button" class="button" name="$id_name" id="image_button" value="Add Image(s)" />';

		// If a file was uploaded, display it below the input.
		$html .= '<p>';
		if ( 'file' === $type ) {

			$file_type   = wp_check_filetype( $file );
			$image_types = array(
				'jpeg',
				'jpg',
				'bmp',
				'gif',
				'png',
			);
			if ( isset( $file ) ) {
				if ( in_array( $file_type['ext'], $image_types ) ) {
					// Display the image.
					$html .= '<img src="' . esc_url( $file ) . '" alt="" style="max-width: 400px;" class="img-' . esc_attr( $id_name ) . '" />';
				} else {
					$html .= '<a href="' . esc_url( $file ) . '">' . esc_html( $file ) . '</a>';
				}
			}
		}
		$html .= '</p>';

		return $html;
	}

	private function render_radio_form_field( $id_name, $inputs, $name, $meta ) {
		// Filter through them, and create options.
		if ( '' != $name ) {
			$radio = '<br/>';
		} else {
			$radio = '';
		}
		foreach ( $inputs[ $name ][1] as $option ) {
			/*
			 * If what's stored in the db is equal to the
			 * current value in the foreach, that should
			 * be the selected one
			 */

			if ( isset( $meta[ $id_name ] ) && $meta[ $id_name ][0] == $option ) {
				$set_checked = 'checked';
			} else {
				$set_checked = '';
			}
			$radio .= '<input type="radio" name="' . esc_attr( $id_name ) . '" value="' . esc_attr( $option ) . '" ' . esc_attr( $set_checked ) . '> ' . esc_html( $option ) . '<br/>';
		}
		$taxarray   = $this->getTax();
		$taxarray[] = $id_name;
		$this->setTax( $taxarray );

		return $radio;
	}

	private function render_checkbox_form_field( $id_name, $name, $esc_value, $type ) {

		$checked = ( 'checkbox' == $type && ! empty( $esc_value ) ? 'checked' : '' );
		$html    = "<input type='checkbox' name='$id_name' value='$name' $checked />";

		return $html;
	}

	private function render_link_form_field( $inputs, $name, $meta, $id_name, $id_names = [] ) {
		$link_array = $inputs[ $name ][1];
		$link_array = array_flip( $link_array );
		$html       = '';

		// Filter through them, and create options.
		foreach ( $link_array as $info => $option ) {
			// @todo True false are the same. Why? Research and fix this.
			$option = ( '1' === $meta[ $id_name ][0] ? $meta[ $id_name ][1] : $meta[ $id_name ][1] );
			$option = ( '1' === $option ? $meta[ $id_name ][1] : $meta[ $id_name ][0] );

			if ( 'Link' === $info ) {
				$html .= "<label>$info: <input type='text' name='$id_names[1]' value='$option' class='slimfat' /></label>";
			} elseif ( 'person_name' === $info ) {
				$html .= "<label>Name: <input type='text_name' name='$id_names[0]' value='$option' class='slimfat' /></label>";
			}
		}

		return $html;
	}

	private function render_text_name_form_field( $id_name, $esc_value, $postID ) {
		$file = get_post_meta( $postID, $id_name, true );
		$html = ( isset( $file ) && ! empty( $file ) ) ?
			"<input type='text' name='$id_name' id='imageurls-$id_name' value='$esc_value' class='widefat' /><input type='button' class='button' name='$id_name' id='edit_image_button-$id_name' value='Edit Image(s)' /><br /><div id='admin-gallery'>" .
			do_shortcode(
				str_replace(
					']',
					" link='none']",
					get_post_meta(
						get_the_ID(),
						'gallery_image(s)',
						true
					)
				)
			) . '</div>'
			: "<input type='text' name='$id_name' id='imageurls-$id_name' value='$esc_value' class='widefat' /><input type='button' class='button' name='$id_name' id='image_button-$id_name' value='Add Image(s)' /><div id='admin-gallery'>" .
			  do_shortcode(
				  str_replace(
					  ']',
					  " link='none']",
					  get_post_meta(
						  get_the_ID(),
						  'gallery_image(s)',
						  true
					  )
				  )
			  ) . '</div>';

		return $html;
	}

	private function kses_allowed_html( $type ) {
		$allowed_html         = [ ];
		$default_allowed_html = [
			'input' => [
				'id'          => [ ],
				'class'       => [ ],
				'type'        => [ ],
				'name'        => [ ],
				'value'       => [ ],
				'style'       => [ ],
				'checked'     => [ ],
				'placeholder' => [ ],
				'min'		  => [ ],
				'step'		  => [ ],
			],
		];

		if ( 'select' === $type ) {
			$allowed_html = [
				'select' => [
					'id'      => [ ],
					'class'   => [ ],
					'type'    => [ ],
					'name'    => [ ],
					'value'   => [ ],
					'checked' => [ ],
				],
				'option' => [
					'id'       => [ ],
					'class'    => [ ],
					'type'     => [ ],
					'name'     => [ ],
					'value'    => [ ],
					'selected' => [ ],
				],
				'br'     => [ ],
			];
		}
		if ( 'textWithDescription' === $type ) {
			$allowed_html = [
				'p'  => [
					'id'    => [ ],
					'class' => [ ],
				],
				'br' => [ ],
			];
		}
		if ( 'textarea' === $type ) {
			$allowed_html = [
				'textarea' => [
					'class'       => [ ],
					'name'        => [ ],
					'rows'        => [ ],
					'placeholder' => [ ],
				],
				'p'  => [
					'id'    => [ ],
					'class' => [ ],
				],
				'br' => [ ],
			];
		}
		if ( 'currency' === $type ) {
			$allowed_html = [
				'input' => [
					'id'          => [ ],
					'class'       => [ ],
					'type'        => [ ],
					'name'        => [ ],
					'value'       => [ ],
					'style'       => [ ],
					'placeholder' => [ ],
					'min'		  => [ ],
					'step'		  => [ ],
				],
				'p'  => [
					'id'    => [ ],
					'class' => [ ],
				],
			];
		}
		if ( 'image' === $type ) {
			$allowed_html = [
				'input' => [
					'id'          => [ ],
					'class'       => [ ],
					'type'        => [ ],
					'name'        => [ ],
					'value'       => [ ],
					'style'       => [ ],
					'placeholder' => [ ],
					'min'		  => [ ],
					'step'		  => [ ],
				],
				'p'  => [
					'id'    => [ ],
					'class' => [ ],
				],
				'a' => [
					'href' => [ ],
					'id' => [ ],
					'title' => [ ],
					'data-uploader_title' => [ ],
					'data-uploader_button_text' => [ ],
				],
				'img' => [
					'src' => [ ],
					'style' => [ ],
				],

			];
		}

		$allowed_html = $default_allowed_html + $allowed_html;

		return $allowed_html;
	}

	private function display_html( $type, $name, $lookup ) {
		$type = is_array( $type ) ? $type['field_type'] : $type;
		$html = $lookup[ $type ];
		if ( 'checkbox' == $type ) {
			echo "<p>" . wp_kses( $html, $this->kses_allowed_html( $type ) ) . "<label>" . esc_html( ucwords( $name ) ) . "</label></p>";
		} elseif ( 'link' === $type ) {
			echo "<div class='meta_subheading'><h3><span>" . esc_html( ucwords( $name ) ) . "</span></h3>" . wp_kses( $html, $this->kses_allowed_html( $type ) ) . "</div>";
		} elseif ( 'wpeditor' === $type ) {
			echo "<div class='meta_subheading'><h3><span>" . esc_html( ucwords( $name ) ) . "</span></h3>" . $html . "</div>";
		} elseif ( 'image' === $type ) {
			echo $html;
		} else {
			$labelname  = ( '' != $name ? ucwords( $name ) . ':' : '' );
			$form_field = '<p><label>' . esc_html( $labelname ) . '</label><br>' . wp_kses( $html, $this->kses_allowed_html( $type ) ) . '</p>';
			echo $form_field;
		}
	}

	/**
	 * When a post saved/updated in the database, this methods updates the meta
	 * box params in the db as well.
	 *
	 * @param integer $post_id The post ID.
	 */
	public function save_post( $post_id ) {
		// Only do the following if we physically submit the form,
		// and now when autosave occurs.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		global $post;
		$metaboxID   = strtolower( str_replace( ' ', '_', $this->title ) );
		$metaboxArgs = array( $this->form_fields );

		$inputs = $metaboxArgs[0];

		// For each form field specified, we need to create the necessary markup.
		// $name = Label, $type = the type of input to create.
		foreach ( $inputs as $name => $type ) {
			$id_name = $metaboxID . '_' .
			           strtolower(
				           str_replace(
					           ' ',
					           '_',
					           $name
				           )
			           );
			if ( is_array( $type ) && array_key_exists( 'js', $type ) ) {
				$this->js = true;
			}
			if ( is_array( $type ) && array_key_exists( 'allowhtml', $type ) ) {
				$this->allowhtml = true;
			}
			$taxarray   = $this->getTax();
			$taxarray[] = $id_name;
			$this->setTax( $taxarray );
		}

		$post_types = array( $this->post_type_name );     // Limit meta box to certain post types.
		$post_type  = get_post_type();
		if ( ! in_array( $post_type, $post_types ) ) {
			return;
		}
		if (
			! empty( $_POST ) && check_admin_referer(
				plugin_basename( __FILE__ ),
				'BP_nonce'
			)
		) {
			// Get all the form fields that were saved in the session,
			// and update their values in the db.
			if ( $this->getTax() ) {

				foreach ( $this->getTax() as $form_name ) {
					$formname = isset($_FILES[ $form_name ]) ? $_FILES[ $form_name ] : '';
					$sanitisedFileName = sanitize_file_name( wp_unslash( $formname ) );
					if ( ! empty( $sanitisedFileName ) ) {

						if ( ! empty( $sanitisedFileName['tmp_name'] ) ) {

							list( $width, $height ) = getimagesize( sanitize_file_name( wp_unslash( $_FILES[ $form_name ]['tmp_name'] ) ) );

							if ( $width > $height ) {
								$orientation = 'horizontal';
							} else {
								$orientation = 'verticalOrSquare';
							}

							if ( isset( $width ) ) {
								if ( 'verticalOrSquare' != $orientation ) {
									update_post_meta(
										$post->ID,
										'images_orientation',
										$orientation
									);
								} else {
									update_post_meta(
										$post->ID,
										'images_orientation',
										$orientation
									);
								}
							}

							$upload = wp_upload_bits(
								sanitize_file_name( wp_unslash( $_FILES[ $form_name ]['name'] ) ),
								null,
								file_get_contents( sanitize_file_name( wp_unslash( $_FILES[ $form_name ]['tmp_name'] ) ) )
							);

							if ( isset( $upload['error'] ) && 0 != $upload['error'] ) {
								wp_die( 'There was an error uploading your file. The error is: ' . esc_html( $upload['error'] ) );
							} else {
								update_post_meta(
									$post->ID,
									$form_name,
									$upload['url']
								);
							}
						}
					} else {
						/*
						// Make better. Have to do this, because I can't figure
						// out a better way to deal with checkboxes. If deselected,
						// they won't be represented here, but I still need to
						// update the value to false to blank in the table. Hmm...
						*/
						if ( ! isset( $_POST[ $form_name ] ) ) {
							$_POST[ $form_name ] = '';
						}

						if ( isset( $post->ID ) ) {
							if ( $this->js ) {
								update_post_meta(
									$post->ID,
									$form_name,
									$_POST[ $form_name ]
								);
							} elseif ( $this->allowhtml ) {
								update_post_meta(
									$post->ID,
									$form_name,
									$_POST[ $form_name ]
								);
							} else {
								update_post_meta(
									$post->ID,
									$form_name,
									// This should be sanitized
									sanitize_text_field( wp_unslash( $_POST[ $form_name ] ) )
								);
							}
						}
					}
				}
				// @TODO: Commented out 2015-09 Evaluate and remove completely on next update.
				// $this->tax = array();
				// $this->setTax( $this->tax );
			}
		}
	}
}
